/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>

void halt() __attribute__((noinline));

void halt()
{
	do {} while(1);
}

int main()
{
	uint16_t data;
	flash_t flash;
	
	intc_init();
	flash_init(&flash);
	uart_lite_init();
	
	PrintString("Write unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA);  /* write unlock cycle 1 */
	PrintString("Write unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055);  /* write unlock cycle 2 */
	PrintString("Write setup command\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x0080);  /* write setup command */
	PrintString("Write additional unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA);  /* write additional unlock cycle 1 */
	PrintString("Write additional unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055);  /* write additional unlock cycle 2 */
	PrintString("Write chip erase command\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x0010);  /* write chip erase command */
	
	PrintString("Verifying sectors");
	
	uint32_t sector;
	uint32_t num_sectors = flash.size / flash.sector_size;
	uint32_t offset;
	for(sector = 0; sector < num_sectors; sector++)
	{
		uint32_t sector_offset = sector * flash.sector_size;
		for(offset = 0; offset < flash.sector_size; offset += flash.word_size)
		{
			do
			{
				data = read_little16((uint16_t *) flash.data + ((sector_offset + offset) / flash.word_size));
			}
			while(data != 0xffff);
		}
		PrintString(".");
	}
	PrintString("\nDone\n");
	PrintString("Test is finished\n");
	halt();
	return 0;
}
