/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "flash.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>

void halt() __attribute__((noinline));

void halt()
{
	do {} while(1);
}

int main()
{
	uint32_t pa = 0x123456;
	uint16_t data = 0x4321;
	uint16_t written_data;
	flash_t flash;
	
	intc_init();
	flash_init(&flash);
	uart_lite_init();
	
	PrintString("Write unlock cycle 1\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00AA);  /* write unlock cycle 1 */
	PrintString("Write unlock cycle 2\n");
	write_little16((uint16_t *) flash.data + 0x2AA, 0x0055);  /* write unlock cycle 2 */
	PrintString("Write program setup command\n");
	write_little16((uint16_t *) flash.data + 0x555, 0x00A0);  /* write program setup command */
	PrintString("Write data to be programmed\n");
	write_little16((uint16_t *) flash.data + pa, data);       /* write data to be programmed */
	
	PrintString("Verify programmed data: ");
	written_data = read_little16((uint16_t *) flash.data + pa);  /* read back data */
	
	PrintString((written_data == data) ? "OK" : "FAILED");
	PrintString("\n");
	PrintString("Test is finished\n");
	halt();
	return 0;
}
