/***************************************************************************
 *                                                                         *
 *   copyright        : (C) 2011 Commissariat a l'Energie Atomique (CEA)   *
 *   author           : Gilles Mouchard                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "timer.h"
#include "io.h"
#include "uart_lite.h"
#include "intc.h"
#include <inttypes.h>



timer_t *timer = 0;

void timer_irq_handler()
{
	if(timer)
	{
		if(read_big32(&timer->tcsr0) & TCSR0_T0INT)
		{
			write_big32(&timer->tcsr0, read_big32(&timer->tcsr0) | TCSR0_T0INT); // acknowlegde interrupt in timer
		}
	}
}

#define PWM_PERIOD 400 /* cycles */
#define PWM_DUTY 60 /* cycles */

int main()
{
	intc_init();    // Map in memory and initialize intc
	uart_lite_init();
	timer = timer_init(); // Map in memory and initialize timer
	register_irq_handler(TIMER_IRQ, timer_irq_handler);
	
	write_big32(&timer->tlr0, PWM_PERIOD - 2);

	write_big32(&timer->tlr1, PWM_DUTY - 2);

	write_big32(&timer->tcsr0, TCSR0_LOAD0); // Move TLR0 into TCR0
	write_big32(&timer->tcsr1, TCSR1_LOAD1); // Move TLR1 into TCR1
	write_big32(&timer->tcsr0, TCSR0_UDT0 | TCSR0_ARHT0 | TCSR0_PWMA0); // down counter, pwm mode, enable interrupt signal, autoreload timer counter
	write_big32(&timer->tcsr1, TCSR1_ENALL | TCSR1_UDT1 | TCSR1_ARHT1 | TCSR1_PWMB0); // enabling all timers, down counter, pwm mode, enable interrupt signal, autoreload timer counter
	
	enable_irq(TIMER_IRQ);

	PrintString("You should observe a PWM output with a period of ");
	PrintUInt32(PWM_PERIOD * TIMER_CYCLE_TIME);
	PrintString(" ns, and a duty cycle of ");
	PrintUInt32(PWM_DUTY * TIMER_CYCLE_TIME);
	PrintString(" ns\n");
	return 0;
}
